@@ -80,7 +80,7 @@ static Bool
 G80GetRec(ScrnInfoPtr pScrn)
 {
     if(pScrn->driverPrivate == NULL)
-        pScrn->driverPrivate = xcalloc(sizeof(G80Rec), 1);
+        pScrn->driverPrivate = calloc(sizeof(G80Rec), 1);
 
     return (pScrn->driverPrivate != NULL);
 }
@@ -88,7 +88,7 @@ G80GetRec(ScrnInfoPtr pScrn)
 static void
 G80FreeRec(ScrnInfoPtr pScrn)
 {
-    xfree(pScrn->driverPrivate);
+    free(pScrn->driverPrivate);
     pScrn->driverPrivate = NULL;
 }
 
@@ -107,7 +107,11 @@ G80ResizeScreen(ScrnInfoPtr pScrn, int width, int height)
     pScrn->virtualY = height;
 
     /* Can resize if XAA is disabled or EXA is enabled */
-    if(!pNv->xaa || pNv->exa) {
+    if(
+#ifdef HAVE_XAA_H
+       !pNv->xaa ||
+#endif
+       pNv->exa) {
         (*pScrn->pScreen->GetScreenPixmap)(pScrn->pScreen)->devKind = pitch;
         pScrn->displayWidth = pitch / (pScrn->bitsPerPixel / 8);
 
@@ -263,7 +267,7 @@ G80PreInit(ScrnInfoPtr pScrn, int flags)
 
     /* Process options */
     xf86CollectOptions(pScrn, NULL);
-    if(!(pNv->Options = xalloc(sizeof(G80Options)))) goto fail;
+    if(!(pNv->Options = malloc(sizeof(G80Options)))) goto fail;
     memcpy(pNv->Options, G80Options, sizeof(G80Options));
     xf86ProcessOptions(pScrn->scrnIndex, pScrn->options, pNv->Options);
 
@@ -429,10 +433,10 @@ G80PreInit(ScrnInfoPtr pScrn, int flags)
     if(!pNv->NoAccel) {
         switch(pNv->AccelMethod) {
         case XAA:
-            if(!xf86LoadSubModule(pScrn, "xaa")) goto fail;
+            if(!xf86LoadSubModule(pScrn, "xaa")) pNv->NoAccel = 1;
             break;
         case EXA:
-            if(!xf86LoadSubModule(pScrn, "exa")) goto fail;
+            if(!xf86LoadSubModule(pScrn, "exa")) pNv->NoAccel = 1;
             break;
         }
     }
@@ -495,16 +499,18 @@ ReleaseDisplay(ScrnInfoPtr pScrn)
 }
 
 static Bool
-G80CloseScreen(int scrnIndex, ScreenPtr pScreen)
+G80CloseScreen(CLOSE_SCREEN_ARGS_DECL)
 {
-    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+    ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
     G80Ptr pNv = G80PTR(pScrn);
 
     if(pScrn->vtSema)
         ReleaseDisplay(pScrn);
 
+#ifdef HAVE_XAA_H
     if(pNv->xaa)
         XAADestroyInfoRec(pNv->xaa);
+#endif
     if(pNv->exa) {
         if(pNv->exaScreenArea) {
             exaOffscreenFree(pScreen, pNv->exaScreenArea);
@@ -530,14 +536,14 @@ G80CloseScreen(int scrnIndex, ScreenPtr pScreen)
     pScrn->vtSema = FALSE;
     pScreen->CloseScreen = pNv->CloseScreen;
     pScreen->BlockHandler = pNv->BlockHandler;
-    return (*pScreen->CloseScreen)(scrnIndex, pScreen);
+    return (*pScreen->CloseScreen)(CLOSE_SCREEN_ARGS);
 }
 
 static void
-G80BlockHandler(int i, pointer blockData, pointer pTimeout, pointer pReadmask)
+G80BlockHandler(BLOCKHANDLER_ARGS_DECL)
 {
-    ScreenPtr pScreen = screenInfo.screens[i];
-    ScrnInfoPtr pScrnInfo = xf86Screens[i];
+    SCREEN_PTR(arg);
+    ScrnInfoPtr pScrnInfo = xf86ScreenToScrn(pScreen);
     G80Ptr pNv = G80PTR(pScrnInfo);
 
     if(pNv->DMAKickoffCallback)
@@ -546,7 +552,7 @@ G80BlockHandler(int i, pointer blockData, pointer pTimeout, pointer pReadmask)
     G80OutputResetCachedStatus(pScrnInfo);
 
     pScreen->BlockHandler = pNv->BlockHandler;
-    (*pScreen->BlockHandler) (i, blockData, pTimeout, pReadmask);
+    (*pScreen->BlockHandler) (BLOCKHANDLER_ARGS);
     pScreen->BlockHandler = G80BlockHandler;
 }
 
@@ -738,7 +744,7 @@ G80InitHW(ScrnInfoPtr pScrn)
 }
 
 static Bool
-G80ScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
+G80ScreenInit(SCREEN_INIT_ARGS_DECL)
 {
     ScrnInfoPtr pScrn;
     G80Ptr pNv;
@@ -747,7 +753,7 @@ G80ScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     BoxRec AvailFBArea;
 
     /* First get the ScrnInfoRec */
-    pScrn = xf86Screens[pScreen->myNum];
+    pScrn = xf86ScreenToScrn(pScreen);
     pNv = G80PTR(pScrn);
 
     pScrn->vtSema = TRUE;
@@ -850,6 +856,7 @@ G80ScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 
     xf86DPMSInit(pScreen, xf86DPMSSet, 0);
 
+#ifdef HAVE_XAA_H
     /* Clear the screen */
     if(pNv->xaa) {
         /* Use the acceleration engine */
@@ -857,7 +864,9 @@ G80ScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
         pNv->xaa->SubsequentSolidFillRect(pScrn,
             0, 0, pScrn->displayWidth, pNv->offscreenHeight);
         G80DmaKickoff(pNv);
-    } else {
+    } else
+#endif
+    {
         /* Use a slow software clear path */
         memset(pNv->mem, 0, pitch * pNv->offscreenHeight);
     }
@@ -885,32 +894,35 @@ G80ScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 }
 
 static void
-G80FreeScreen(int scrnIndex, int flags)
+G80FreeScreen(FREE_SCREEN_ARGS_DECL)
 {
-    G80FreeRec(xf86Screens[scrnIndex]);
+    SCRN_INFO_PTR(arg);
+    G80FreeRec(pScrn);
 }
 
 static Bool
-G80SwitchMode(int scrnIndex, DisplayModePtr mode, int flags)
+G80SwitchMode(SWITCH_MODE_ARGS_DECL)
 {
-    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+    SCRN_INFO_PTR(arg);
     return xf86SetSingleMode(pScrn, mode, RR_Rotate_0);
 }
 
 static void
-G80AdjustFrame(int scrnIndex, int x, int y, int flags)
+G80AdjustFrame(ADJUST_FRAME_ARGS_DECL)
 {
 }
 
 static Bool
-G80EnterVT(int scrnIndex, int flags)
+G80EnterVT(VT_FUNC_ARGS_DECL)
 {
-    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+    SCRN_INFO_PTR(arg);
     G80Ptr pNv = G80PTR(pScrn);
 
     /* Reinit the hardware */
+#ifdef HAVE_XAA_H
     if(pNv->xaa)
         G80InitHW(pScrn);
+#endif
 
     if(!AcquireDisplay(pScrn))
         return FALSE;
@@ -919,9 +931,9 @@ G80EnterVT(int scrnIndex, int flags)
 }
 
 static void
-G80LeaveVT(int scrnIndex, int flags)
+G80LeaveVT(VT_FUNC_ARGS_DECL)
 {
-    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+    SCRN_INFO_PTR(arg);
 
     ReleaseDisplay(pScrn);
 }
